!$Id: init_obscovar_pdaf.F90 267 2019-11-28 12:50:18Z lnerger $
!BOP
!
! !ROUTINE: init_obscovar_pdaf --- Initialize observation error covariance matrix
!
! !INTERFACE:
SUBROUTINE init_obscovar_pdaf(step, dim_obs, dim_obs_p, covar, m_state_p, &
     isdiag)

! !DESCRIPTION:
! User-supplied routine for PDAF.
! Used in the filters: EnKF
!
! The routine is called during the analysis
! step when an ensemble of observations is
! generated by PDAF\_enkf\_obs\_ensemble. 
! It has to initialize the global observation 
! error covariance matrix.
!
! The routine is called by all filter processes.
! 
! Version for the Lorenz63 model without parallelization,
! diagonal observation error covariance matrix and
! constant variance.
!
! !REVISION HISTORY:
! 2004-11 - Lars Nerger - Initial code
! Later revisions - see svn log
!
! !USES:
  USE mod_assimilation, &
       ONLY: rms_obs

  IMPLICIT NONE

! !ARGUMENTS:
  INTEGER, INTENT(in) :: step                 ! Current time step
  INTEGER, INTENT(in) :: dim_obs              ! Dimension of observation vector
  INTEGER, INTENT(in) :: dim_obs_p            ! PE-local dimension of obs. vector
  REAL, INTENT(out) :: covar(dim_obs,dim_obs) ! Observation error covar. matrix
  REAL, INTENT(in) :: m_state_p(dim_obs_p)    ! Observation vector
  LOGICAL, INTENT(out) :: isdiag              ! Whether matrix R is diagonal

! !CALLING SEQUENCE:
! Called by: PDAF_enkf_obs_ensemble    (as U_init_obscovar)
!EOP

! *** local variables ***
  INTEGER :: i          ! Index of observation component
  REAL :: variance_obs  ! ariance of observations


! **********************
! *** INITIALIZATION ***
! **********************

  variance_obs = rms_obs ** 2

  covar(:, :) = 0.0


! *************************************
! ***   Initialize covariances      ***
! *************************************

! *** We assume uncorrelated Measurements here, thus R is diagonal

  DO i = 1, dim_obs
     covar(i, i) = variance_obs
  ENDDO

  ! The matrix is diagonal
  ! This setting avoids the computation of the SVD of COVAR
  ! in PDAF_enkf_obs_ensemble
  isdiag = .TRUE.

END SUBROUTINE init_obscovar_pdaf
